/*
 * $Id: MainLoop.as,v 1.4 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.Sprite;
  import flash.display.Bitmap;
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import flash.events.Event;
  import org.cove.ape.APEngine;
  import org.cove.ape.Group;
  import org.cove.ape.Vector;
  import abagames.util.KeyState;

  /**
   * Main loop that updates a game's state and handles actor pools.
   */
  public class MainLoop {
    private static const TITLE:int = 0;
    private static const IN_GAME:int = 1;
    private var rootSprite:Sprite;
    private var buffer:BitmapData;
    private var screen:Bitmap;
    private var keyState:KeyState;
    private var field:Field;
    private var player:Player;
    private var balls:BallPool;
    private var solidBalls:SolidBallPool;
    private var fragments:FragmentPool;
    private var cords:CordPool;
    private var mines:MinePool;
    private var energies:EnergyPool;
    private var smallEnergies:SmallEnergyPool;
    private var floors:FloorPool;
    private var particles:ParticlePool;
    private var stage:Stage;
    private var title:Title;
    private var sound:SoundHandler;
    private var groupCollideInternal:Group;
    private var group:Group;
    private var mode:int;
    private var pPressed:Boolean;
    private var isPaused:Boolean;
    private var cnt:int;
    private var store:Store;

    public function init(rootSprite:Sprite, keyState:KeyState, buffer:BitmapData, screen:Bitmap, sounds:Array):void {
      this.rootSprite = rootSprite;
      this.keyState = keyState;
      this.buffer = buffer;
      this.screen = screen;
      groupCollideInternal = new Group(true);
      group = new Group;
      APEngine.init(0.25);
      APEngine.container = rootSprite;
      APEngine.addMasslessForce(new Vector(0, Field.GRAVITY));
      APEngine.damping = 0.99;
      APEngine.addGroup(groupCollideInternal);
      APEngine.addGroup(group);
      groupCollideInternal.addCollidable(group);
      sound = new SoundHandler(sounds);
      store = new Store;
      field = new Field(buffer, groupCollideInternal);
      particles = new ParticlePool(512, buffer, field);
      fragments = new FragmentPool(64, buffer, field, group);
      cords = new CordPool(32, buffer, field, groupCollideInternal);
      player = new Player(buffer, keyState, field, fragments, particles, groupCollideInternal, sound, this);
      energies = new EnergyPool(4, buffer, field, player, particles, groupCollideInternal);
      smallEnergies = new SmallEnergyPool(64, buffer, field, player, particles, group);
      mines = new MinePool(16, buffer, field, player, fragments, particles, smallEnergies, sound, groupCollideInternal);
      balls = new BallPool(64, buffer, field, player, fragments, particles, smallEnergies, sound, groupCollideInternal);
      solidBalls = new SolidBallPool(16, buffer, field, player, fragments, particles, sound, groupCollideInternal);
      floors = new FloorPool(32, buffer, field, cords, groupCollideInternal);
      stage = new Stage(particles, player, balls, solidBalls, cords, mines, energies, floors, field);
      title = new Title(buffer, field, keyState, player, store, balls, solidBalls, mines, energies, this);
      player.setParams(stage);
      Letter.init(buffer, field);
      pPressed = false;
      isPaused = false;
      startTitle();
    }

    public function startTitle():void {
      mode = TITLE;
      isPaused = false;
      initPools();
      field.initTitle();
      player.initTitle();
      stage.init();
      title.init();
      sound.stopBgm();
    }

    public function startInGame():void {
      mode = IN_GAME;
      cnt = 0;
      initPools();
      field.init();
      player.init();
      stage.init();
      sound.startBgm();
    }

    public function startGameover():void {
      store.storeScore(player.height);
      sound.fadeoutBgm();
    }

    private function initPools():void {
      particles.clear();
      fragments.clear();
      balls.clear();
      mines.clear();
      energies.clear();
      smallEnergies.clear();
      floors.clear();
      cords.clear();
    }

    public function update():void {
      buffer.fillRect(buffer.rect, 0);
      switch (mode) {
      case TITLE:
        updateActors();
        title.update();
        break;
      case IN_GAME:
        cnt++;
        if (keyState.esc) {
          startTitle();
          break;
        }
        if (keyState.p) {
          if (!pPressed) {
            pPressed = true;
            isPaused = !isPaused;
          }
        } else {
          pPressed = false;
        }
        if (isPaused) {
          if (cnt % 30 < 15)
            Letter.drawString("PAUSE", 100, 100, 0.75);
          break;
        }
        updateActors();
        break;
      }
      sound.update();
    }

    private function updateActors():void {
      APEngine.step();
      field.update();
      stage.update();
      if (player.hasCollision)
        player.drawBlur();
      particles.update();
      fragments.update();
      cords.update();
      floors.update();
      balls.update();
      solidBalls.update();
      mines.update();
      energies.update();
      smallEnergies.update();
      player.update();
    }

    public function playerDestroyed():void {
      energies.clear();
      smallEnergies.clear();
      solidBalls.destroy();
      balls.destroy();
      mines.destroy();
    }
  }
}
